#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import copy
import testtools
from testtools import matchers

from cyborgclient.tests.unit import utils
from cyborgclient.v1 import deployables


DEPLOYABLE1 = {
    "instance_uuid": "", "assignable": True,
    "vendor": "test_vendor1",
    "parent_uuid": "1859ef92-90c9-462e-ba78-3fd8295aa390",
    "links": [
        {"href": "http://127.0.0.1:6666/v1/deployables/fake_uuid",
         "rel": "self"
         },
        {"href": "http://127.0.0.1:6666/deployables/fake_uuid",
         "rel": "bookmark"}],
    "updated_at": "2018-07-27T13:07:14+00:00",
    "interface_type": "pci",
    "uuid": "1859ef92-90c9-462e-ba78-3fd8295aa390",
    "name": "test_name1", "host": "host_test1",
    "version": "1", "board": "test_board1",
    "address": "test_addr1", "created_at": None,
    "type": "pf", "availability": "1",
    "root_uuid": "1859ef92-90c9-462e-ba78-3fd8295aa390"
}
DEPLOYABLE2 = {
    "instance_uuid": None, "assignable": False,
    "vendor": "test_vendor2",
    "parent_uuid": "5a7dfaf9-7f4e-42d0-bfac-b2f464110d9f",
    "links": [
        {"href": "http://127.0.0.1:6666/v1/deployables/fake_uuid",
         "rel": "self"},
        {"href": "http://127.0.0.1:6666/deployables/fake_uuid",
         "rel": "bookmark"}],
    "updated_at": "2018-07-27T19:51:18+00:00",
    "interface_type": "mdev",
    "uuid": "5a7dfaf9-7f4e-42d0-bfac-b2f464110d9f",
    "name": "test_name2", "host": "host_test2", "version": "2",
    "board": "test_board2", "address": "test_addr2",
    "created_at": None, "type": "vf", "availability": "1",
    "root_uuid": "5a7dfaf9-7f4e-42d0-bfac-b2f464110d9f"
}
ALLOC_DEPLOYABLE1 = {
    "instance_uuid": None, "assignable": False,
    "vendor": "test_vendor2",
    "parent_uuid": "5a7dfaf9-7f4e-42d0-bfac-b2f464110d9f",
    "links": [
        {"href": "http://127.0.0.1:6666/v1/deployables/fake_uuid",
         "rel": "self"},
        {"href": "http://127.0.0.1:6666/deployables/fake_uuid",
         "rel": "bookmark"}],
    "updated_at": "2018-07-27T19:51:18+00:00",
    "interface_type": "mdev",
    "uuid": "5a7dfaf9-7f4e-42d0-bfac-b2f464110d9f",
    "name": "test_name2", "host": "host_test2", "version": "2",
    "board": "test_board2", "address": "test_addr2",
    "created_at": None, "type": "vf", "availability": "1",
    "root_uuid": "5a7dfaf9-7f4e-42d0-bfac-b2f464110d9f"
}
fake_responses = {
    '/v1/accelerators/deployables':
    {
        'GET': (
            {},
            {'deployables': [DEPLOYABLE1, DEPLOYABLE2]},
        )
    },
    '/v1/accelerators/deployables/%s' % ALLOC_DEPLOYABLE1["uuid"]:
    {
        'PATCH': (
            {"instance_uuid": "fake_instance_uuid"},
            ALLOC_DEPLOYABLE1,
        ),
    },
    '/v1/accelerators/deployables?filters.field=limit&filters.value=2':
    {
        'GET': (
            {},
            {'deployables': [DEPLOYABLE1, DEPLOYABLE2]},
        )
    },
    '/v1/accelerators/deployables?filters.field=marker&filters.value=%s' %
    DEPLOYABLE1['uuid']:
    {
        'GET': (
            {},
            {'deployables': [DEPLOYABLE1, DEPLOYABLE2]},
        )
    }
}


class DeployableManagerTest(testtools.TestCase):

    def setUp(self):
        super(DeployableManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = deployables.DeployableManager(self.api)

    def test_deployables_list(self):
        deployables = self.mgr.list()
        expect = [
            ('GET', '/v1/accelerators/deployables', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(deployables, matchers.HasLength(2))

    def _test_deployables_list_with_filters(self, limit=None, marker=None,
                                            sort_key=None, sort_dir=None,
                                            expect=[], **add_filters):
        deployables_filter = self.mgr.list(limit=limit, marker=marker,
                                           sort_key=sort_key,
                                           sort_dir=sort_dir, **add_filters)
        self.assertEqual(expect, self.api.calls)
        self.assertThat(deployables_filter, matchers.HasLength(2))

    def test_deployables_list_with_limit(self):
        expect = [
            ('GET', '/v1/accelerators/deployables?filters.field='
                    'limit&filters.value=2', {}, None),
        ]
        self._test_deployables_list_with_filters(
            limit=2,
            expect=expect)

    def test_deployables_list_with_marker(self):
        expect = [
            ('GET', '/v1/accelerators/deployables?filters.field=marker&'
                    'filters.value=%s' % DEPLOYABLE1['uuid'],
             {}, None),
        ]
        self._test_deployables_list_with_filters(
            marker=DEPLOYABLE1['uuid'],
            expect=expect)

    def test_deployables_allocation(self):
        dep_for_alloc = copy.deepcopy(ALLOC_DEPLOYABLE1)
        self.mgr.allocation(dep_for_alloc["uuid"], "fake_instance_uuid")
        expect = [
            ('PATCH', '/v1/accelerators/deployables/%s' %
             dep_for_alloc["uuid"],
             {}, [{'op': 'replace',
                   'path': '/instance_uuid',
                   'value': 'fake_instance_uuid'}])
        ]
        self.assertEqual(expect, self.api.calls)

    def test_deployables_deallocation(self):
        dep_for_dealloc = copy.deepcopy(ALLOC_DEPLOYABLE1)
        self.mgr.deallocation(dep_for_dealloc["uuid"])
        expect = [
            ('PATCH', '/v1/accelerators/deployables/%s' %
             dep_for_dealloc["uuid"],
             {}, [{'op': 'replace',
                   'path': '/instance_uuid',
                   'value': None}])
        ]
        self.assertEqual(expect, self.api.calls)



#   Copyright (c) 2018 Intel, Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#


"""Cyborg v1 Acceleration accelerator action implementations"""


from osc_lib.command import command
from osc_lib import utils

from cyborgclient.common import utils as cli_utils
from cyborgclient.i18n import _


class ListAccelerator(command.Lister):
    """List all accelerators"""

    def get_parser(self, prog_name):
        parser = super(ListAccelerator, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help=_("List additional fields in output")
        )
        return parser

    def take_action(self, parsed_args):
        acc_client = self.app.client_manager.accelerator

        column_headers = (
            "UUID",
            "Name",
            "Type",
            "Description",
            "Device Type",
            "Capability",
        )
        columns = (
            "uuid",
            "name",
            "type",
            "description",
            "device_type",
            "capability",
        )

        data = acc_client.accelerators.list()
        if not data:
            return (), ()
        column_headers, columns = cli_utils.clean_listing_columns(
            column_headers, columns, data[0])
        formatters = {}

        return (column_headers,
                (utils.get_item_properties(
                    s, columns, formatters=formatters) for s in data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import copy

from openstack import exceptions as sdk_exc
from osc_lib import utils as oscutils

from cyborgclient.common import utils
from cyborgclient import exceptions as exc
from cyborgclient.osc.v2 import device_profile as osc_device_profile
from cyborgclient.tests.unit.osc.v2 import fakes as acc_fakes


class TestDeviceProfile(acc_fakes.TestAccelerator):

    def setUp(self):
        super(TestDeviceProfile, self).setUp()

        self.mock_acc_client = self.app.client_manager.accelerator
        self.mock_acc_client.reset_mock()


class TestDeviceProfileList(TestDeviceProfile):

    def setUp(self):
        super(TestDeviceProfileList, self).setUp()

        self.mock_acc_client.device_profiles.return_value = [
            acc_fakes.FakeAcceleratorResource(
                None,
                copy.deepcopy(acc_fakes.DEVICE_PROFILE),
                loaded=True)
        ]

        self.cmd = osc_device_profile.ListDeviceProfile(self.app, None)

    def test_device_profile_list(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        kwargs = {}

        self.mock_acc_client.device_profiles.assert_called_with(**kwargs)

        collist = (
            'uuid',
            'name',
            'groups',
            'description'
        )
        self.assertEqual(collist, columns)

        datalist = [(
            acc_fakes.device_profile_uuid,
            acc_fakes.device_profile_name,
            acc_fakes.device_profile_groups,
            acc_fakes.device_profile_description,
        ), ]
        self.assertEqual(datalist, list(data))

    def test_device_profile_list_long(self):
        arglist = ['--long']
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        kwargs = {}

        self.mock_acc_client.device_profiles.assert_called_with(**kwargs)

        collist = (osc_device_profile.ListDeviceProfile.columns +
                   osc_device_profile.ListDeviceProfile.detail_cols)
        self.assertEqual(list(collist), list(columns))

        datalist = [(
            acc_fakes.device_profile_uuid,
            acc_fakes.device_profile_name,
            acc_fakes.device_profile_groups,
            acc_fakes.device_profile_description,
            acc_fakes.device_profile_created_at,
            acc_fakes.device_profile_updated_at,
        ), ]
        self.assertEqual(datalist, list(data))


class TestDeviceProfileShow(TestDeviceProfile):

    def setUp(self):
        super(TestDeviceProfileShow, self).setUp()
        self.cmd = osc_device_profile.ShowDeviceProfile(self.app, None)

    def test_device_profile_show_with_name_before_v22(self):
        exc_msg = 'The minimal required API version should be 2.2'
        self.mock_acc_client.get_device_profile.side_effect = \
            sdk_exc.HttpException(details=exc_msg)

        arg_list = [acc_fakes.device_profile_name]
        verify_list = []
        parsed_args = self.check_parser(self.cmd, arg_list, verify_list)
        result = self.assertRaises(exc.NotAcceptable,
                                   self.cmd.take_action,
                                   parsed_args)
        self.assertIn(exc_msg, str(result))

    def test_device_profile_show_with_name(self):
        fake_dp = acc_fakes.FakeAcceleratorResource(
            None,
            copy.deepcopy(acc_fakes.DEVICE_PROFILE),
            loaded=True)
        self.mock_acc_client.get_device_profile.return_value = fake_dp

        arg_list = [acc_fakes.device_profile_name]
        verify_list = []
        parsed_args = self.check_parser(self.cmd, arg_list, verify_list)
        result = self.cmd.take_action(parsed_args)
        columns = (
            "created_at",
            "updated_at",
            "uuid",
            "name",
            "groups",
            "description",
        )
        formatters = {'data': utils.json_formatter}
        expected = oscutils.get_dict_properties(acc_fakes.DEVICE_PROFILE,
                                                columns,
                                                formatters=formatters)
        self.assertEqual(result, (columns, expected))


class TestDeviceProfileCreate(TestDeviceProfile):

    def setUp(self):
        super(TestDeviceProfileCreate, self).setUp()

        fake_dp = acc_fakes.FakeAcceleratorResource(
            None,
            copy.deepcopy(acc_fakes.DEVICE_PROFILE),
            loaded=True)
        self.mock_acc_client.create_device_profile.return_value = fake_dp
        self.mock_acc_client.get_device_profile.return_value = fake_dp
        self.cmd = osc_device_profile.CreateDeviceProfile(self.app, None)

    def test_device_profile_create(self):
        arglist = ['test', '[]']
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        kwargs = {'name': 'test', 'groups': [], 'description': None}

        self.mock_acc_client.create_device_profile.assert_called_with(**kwargs)

        collist = (
            'created_at',
            'updated_at',
            'uuid',
            'name',
            'groups',
            'description'
        )
        self.assertEqual(collist, columns)

        datalist = [
            acc_fakes.device_profile_created_at,
            acc_fakes.device_profile_updated_at,
            acc_fakes.device_profile_uuid,
            acc_fakes.device_profile_name,
            acc_fakes.device_profile_groups,
            acc_fakes.device_profile_description,
        ]
        self.assertEqual(datalist, list(data))



# Copyright (c) 2018 Intel, Inc. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from cyborgclient.common import cliutils as utils
from cyborgclient.i18n import _


@utils.arg('--limit',
           metavar='<limit>',
           type=int,
           help=_('Maximum number of accelerators to return'))
@utils.arg('--sort-key',
           metavar='<sort-key>',
           help=_('Column to sort results by'))
@utils.arg('--sort-dir',
           metavar='<sort-dir>',
           choices=['desc', 'asc'],
           help=_('Direction to sort. "asc" or "desc".'))
@utils.arg('--fields',
           default=None,
           metavar='<fields>',
           help=_('Comma-separated list of fields to display. '
                  'Available fields: uuid, name'
                  )
           )
@utils.arg('--detail',
           action='store_true', default=False,
           help=_('Show detailed information about the accelerators.')
           )
def do_accelerator_list(cs, args):
    """Print a list of accelerators.

    (Deprecated for V1)
    """
    utils.deprecated("v1 APIs for accelerator objects are deprecated.")
